;**********************************************************************
;                                                                     *
;    Filename:	    LS.asm                                            *
;    Date:          06.04.2009                                        *
;    File Version:  V1.1                                              *
;                                                                     *
;    Author:        Christian Batz                                    *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;    12F627a.lkr                                                      *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;    Lichtschrankensystem zur Richtungsanalyse und Personenzhlung    *
;    Signalausgang: RA0                                               *
;    Relaisausgang: RA3                                               *
;    Signaleingnge (LS0/LS1): RB4/RB5                                *
;    7-Seg Ausgnge: RA7->a, RA6->b, RA4->c                           *
;                    RB3->d, RB2->e, RB1->f, RB0->g                   *
;    Helligkeitssensoreingnge: RA1, RA2                              *
;    Manuelle Zhlung: RB6 -> 0 or RB7 -> 0                           *
;    Umschaltung Anzeige: RB6 and RB7 == 0                            *
;    Sequenz: 00-01-(11)-10-00 bzw. 00-10-(11)-01-00                  *
;                                                                     *
;**********************************************************************

	list      p=16f627A					; list directive to define processor
	#include <p16F627A.inc>				; processor specific variable definitions

	errorlevel  -302					; suppress message 302 from list file

	__CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT 

;***** DATA DEFINITIONS BANK0

;***** LS SEQUENCE

MAIN			UDATA		0x20
state			RES			1				; current LS state
track			RES			1				; LS track
track_part		RES			1				; temp track

;***** LS BASIC

ls_count		RES			1				; counter for LS check intervall
LS0_count		RES			1				; counter for LS0 signal puls
LS1_count		RES			1				; counter for LS1 signal puls

;***** TIMING

ms4_scaler		RES			1				; scaler to get 4ms
s1_scaler		RES			1				; scaler to get 1s
m1_scaler		RES			1				; scaler to get 1min
delay_05s		RES			1				; counter to get 0,5s delay

;***** RELAIS

re_delay_min	RES			1				; delay value (min)
re_delay_sec	RES			1				; delay value (sec)

;***** SWITCHES

sw1_deb_count	RES			1				; switch one debounce counter
sw2_deb_count	RES			1				; switch two debounce counter

;***** COUNTING

obj_count_e		RES			1				; object counter 1st digit
obj_count_z		RES			1				; object counter 2nd digit
obj_count_r		RES			1				; object counter rest -> 25599 objs possible
counter			RES			1				; general counter (config mode)
counter_z		RES			1				; general counter 2nd digit (config mode)
counter_max		RES			1				; maximum counter value (config mode)

;***** DISPLAY

disp_value		RES			1				; variable for saving 7-seg code value
left			RES			1				; variable for left display
right			RES			1				; variable for right display

;***** EEPROM

EEPROM			UDATA		0x6a
EE				RES			5				; data stored in EEPROM


;***** VARIABLE DEFINITIONS SHARE

INT_VAR			UDATA_SHR	0x71			; shared RAM
w_temp			RES			1				; variable used for context saving 
status_temp		RES			1				; variable used for context saving
general_a		RES			1				; used for single bits (flags)
general_b		RES			1
temp			RES			1				; for temporary data

;***** DECALARATIONS

seq_a			EQU			b'00011000'		; sequences
seq_b			EQU			b'00100100'

d_0				EQU			b'01111110'		; codes for 0 - 9
d_1				EQU			b'00110000'		; bit 6 means a and bit 0 means g
d_2				EQU			b'01101101'
d_3				EQU			b'01111001'
d_4				EQU			b'00110011'
d_5				EQU			b'01011011'
d_6				EQU			b'01011111'
d_7				EQU			b'01110000'
d_8				EQU			b'01111111'
d_9				EQU			b'01111011'

ls_off			EQU			b'00110110'		; no signal
ls_on			EQU			b'00110111'		; signal

char_S			EQU			b'01011011'
char_E			EQU			b'01001111'
char_L			EQU			b'00001110'
char_I			EQU			b'00000110'
char_d			EQU			b'00111101'

SW_deb_mark		EQU			d'12'			; debounce marker for switches

;***** DEFINITIONS - FLAGS

;** FLAGS

#define			LS0				general_a,0	; LS on or off
#define			LS1				general_a,1
#define			start_up		general_a,2	; start up mode
#define			SW1				general_a,3	; switch last state (on or off)
#define			SW2				general_a,4
#define			config_mode		general_a,5	; configuration mode
#define			obj_state		general_a,6	; obj counter > 0

#define			RE_trigger_flag	general_b,0	; trigger to check relais state
#define			RE_delay_flag	general_b,1	; relais in delay mode

;** EE VARS

#define			config_level	EE			; config level counter
#define			delay_sec		EE+1		; relais delay sec
#define			delay_min		EE+2		; relais delay min
#define			sensitivity		EE+3		; sensitivity of LS (20 = high sensitivity)
#define			light_mode		EE+4		; indoor or outdoor light detection

;***** MACROS

BANK0			macro
				bcf	STATUS,RP0
				bcf	STATUS,RP1
				endm

BANK1			macro
				bsf	STATUS,RP0
				bcf	STATUS,RP1
				endm

BANK2			macro
				bcf	STATUS,RP0
				bsf	STATUS,RP1
				endm

BANK3			macro
				bsf	STATUS,RP0
				bsf	STATUS,RP1
				endm
				

;**********************************************************************

RESET_VECTOR	CODE	0x000				; processor reset vector
				goto	Main				; go to beginning of program
	
;***** GENERAL INT HANDLING

INT_VECTOR		CODE	0x004				; interrupt vector location
				movwf	w_temp				; save off current W register contents
				movfw	STATUS				; move status register into W register
				movwf	status_temp			; save off contents of STATUS register

;** INT HANDLING (COMPARE INT EVERY 500us)

				BANK0						; select bank 0
				btfsc	PIR1,CCP1IF			; compare int
				call	CCP1Int

				movfw	status_temp			; retrieve copy of STATUS register
				movwf	STATUS				; restore pre-isr STATUS register contents
				swapf	w_temp,f
				swapf	w_temp,w			; restore pre-isr W register contents
				retfie						; return from interrupt

;***** CCP1 INT -> TIMING MODULE, LS BASIC HANDLING, DISPLAY UPDATE, RELAIS OFF

CCP1Int			bcf		PIR1,CCP1IF			; clears CCP1 int flag

;*** TIMING MODULE

				decfsz	ms4_scaler,f
				goto	LSHandling
				movlw	d'8'				; 8 x 500us = 4ms
				movwf	ms4_scaler

;** 4MS APPLICATION TIMER

				movfw	sw1_deb_count
				btfss	STATUS,Z
				decf	sw1_deb_count,f		; debouncing switch 1
				movfw	sw2_deb_count
				btfss	STATUS,Z
				decf	sw2_deb_count,f		; debouncing switch 2
				
				movfw	ls_count
				btfss	STATUS,Z			; counter for LS check intervall
				decf	ls_count,f

				movfw	delay_05s			; counter for 0,5s delay
				btfss	STATUS,Z
				decf	delay_05s,f

;*** TIMING MODULE

				decfsz	s1_scaler,f
				goto	LSHandling
				movlw	d'250'				; 250 x 4ms = 1s
				movwf	s1_scaler

;** 1S APPLICATION TIMER

				movfw	re_delay_min
				btfss	STATUS,Z			; relais delay counter sec if min == 0
				goto	Timing1Min
				movfw	re_delay_sec
				btfss	STATUS,Z
				decf	re_delay_sec,f

;** TIMING MODULE

Timing1Min		decfsz	m1_scaler,f
				goto	LSHandling
				movlw	d'60'				; 60 x 1s = 1min
				movwf	m1_scaler

;** 1MIN APPLICATION TIMER

				movfw	re_delay_min		; relais delay counter min
				btfss	STATUS,Z
				decf	re_delay_min,f

;*** LS BASIC HANDLING

LSHandling		movfw	ls_count
				btfss	STATUS,Z			; counter check ?
				goto	LSCounting
				movlw	d'5'				; resets LS intervall counter
				movwf	ls_count
				movfw	sensitivity			; default for LS0 counter (max 20)
				subwf	LS0_count,w			; counter > default ?
				btfss	STATUS,C
				goto	LS0Broken
				bcf		LS0					; LS0 is okay
				goto	CheckLS1
LS0Broken		bsf		LS0					; LS0 is broken

CheckLS1		movfw	sensitivity			; default for LS1 counter (max 20)
				subwf	LS1_count,w			; counter > default ?
				btfss	STATUS,C
				goto	LS1Broken
				bcf		LS1					; LS1 is okay
				goto	FinishCheck
LS1Broken		bsf		LS1					; LS1 is broken

FinishCheck		clrf	LS0_count			; resetting counter
				clrf	LS1_count
		
LSCounting		btfss	PORTB,4
				incf	LS0_count,f			; counts falling edge for LS0
				btfss	PORTB,5
				incf	LS1_count,f			; counts falling edge for LS1

;*** UPDATE DISPLAY

				movlw	b'00101111'
				andwf	PORTA,f				; clears display (PORTA)
				movlw	b'11110000'
				andwf	PORTB,f				; clears display (PORTB)
				movlw	b'00000001'
				xorwf	PORTA,f				; toggles RA0 (1 KHz)
				call	UpdateDisp			; updates 7 segment display
				return

;***** 7 SEGMENT DISPLAY OUTPUT (RB0-RB3, RA4,RA6,RA7) - CONTROL VIA RA0

;** 7 SEGMENT DISPLAY CONVERSION

SevSegCon		clrf	PCLATH
				addwf	PCL,f				; skips certain number of instructions
				retlw	d_0					; returns with correct 7-seg code
				retlw	d_1
				retlw	d_2
				retlw	d_3
				retlw	d_4
				retlw	d_5
				retlw	d_6
				retlw	d_7
				retlw	d_8
				retlw	d_9

;**  DISPLAY MODE

UpdateDisp		btfsc	PORTB,6
				goto	ChkMode				; counter or config mode
				btfss	PORTB,7
				goto	GetLSValues			; gets LS values and display LS status
ChkMode			btfsc	config_mode			; config mode ?
				goto	ConfigMode

;** COUNTER MODE

				movfw	obj_count_e			; gets 1st digit data
				btfss	PORTA,0				; which display to update
				movfw	obj_count_z			; gets 2nd digit data
				call	SevSegCon			; converts 1 digit to 7-seg code
				goto	ShowDisp			; show counter

;** LS MODE

GetLSValues		btfss	PORTA,0				; which display to update
				goto	GetLS1				; update LS0 display
				movlw	ls_off				; default is for LS0 off
				btfss	LS0					; LS0 is off ?
				movlw	ls_on				; LS0 is on
				goto	ShowDisp			; show status
GetLS1			movlw	ls_off				; default for LS1 is off
				btfss	LS1					; LS1 is off ?
				movlw	ls_on				; LS1 is on
				goto	ShowDisp			; show status

;** CONFIG MODE

ConfigMode		btfss	start_up
				goto	CountingMode		; mode for getting config values			
				movfw	right				; output of mode level
				btfss	PORTA,0
				movfw	left
				goto	ShowDisp	
CountingMode	clrf	counter_z			; clears 2nd digit counter
				movfw	counter
				movwf	temp				; copies counter to temp
				movlw	d'10'
ZSub			subwf	temp,f				; gets 2nd digit part by subtracting 10
				btfss	STATUS,C			; result < 0 ?
				goto	ZFinish
				incf	counter_z,f			; increases 2nd digit counter
				goto	ZSub
ZFinish			addwf	temp,f				; undo last step to make temp > 0 -> 1st digit
				movfw	counter_z			; gets 2nd digit
				btfsc	PORTA,0				; which display to update
				movfw	temp				; gets 1st digit
				call	SevSegCon

;** UPDATE DISPLAY (RA7->a, RA6->b, RA4->c, RB3->d, RB2->e, RB1->f, RB0->g)

ShowDisp		movwf	disp_value			; saves 7-seg code for later use
				movlw	b'11110000'			; gets high nibble of PORTB
				andwf	PORTB,w		
				xorlw	b'00001111'			; makes low nibble high
				movwf	temp				; saves result for later use
				movfw	disp_value			; gets current 7-seg code
				andlw	b'00001111'			; gets low nibble
				xorlw	b'11110000'			; makes high nibble high
				andwf	temp,w				; combines PORTB high nibble with 7-seg code low nibble
				movwf	PORTB				; writes on PORTB
				movlw	b'00101111'			; get low nibble of PORTA and RA5
				andwf	PORTA,w	
				xorlw	b'11010000'			; makes high nibble high besides RA5
				movwf	temp				; saves result for later use
				movlw	b'11110000'
				andwf	disp_value,f		; gets high nibble of current 7-seg code
				movlw	b'00010000'
				andwf	disp_value,w		; gets value RA4
				rlf		disp_value,f		; RA5 and RA6 become bit RA6 and RA7
				bcf		disp_value,0		; clears used carry bit
				bcf		disp_value,5		; clears RA5 (input only)
				bsf		disp_value,4		; sets RA4 high
				xorlw	b'11101111'			; makes all bit high besides RA4
				andwf	disp_value,w		; sets RA4
				xorlw	b'00101111'			; makes low nibble and RA5 high
				andwf	temp,w				; combines PORTA low high nibble with 7-seg code high nibble
				movwf	PORTA				; writes on PORTA
				return

;***** CHECKING SWITCHES (RB6 + RB7)

ChkSwitches		call	SwitchOne			; checks change of switch one
				andlw	0x01
				btfsc	STATUS,Z			; change detected ?
				goto	ChkSw2				; no change
				btfsc	config_mode
				goto	IncCounter			; config mode
				call	IncObj				; increases counter
				goto	ChkSw2

IncCounter		movfw	counter_max
				subwf	counter,w			; counter = max ?
				btfsc	STATUS,Z
				goto	CounterOver			; counter overflow
				incf	counter,f			; increases counter
				goto	ChkSw2
CounterOver		clrf	counter				; resets counter

ChkSw2			call	SwitchTwo			; checks change of switch two
				andlw	0x01
				btfsc	STATUS,Z			; change detected ?
				return						; no change
				btfsc	config_mode
				goto	DecCounter			; config mode
				call	DecObj				; decreases counter
				return

DecCounter		movfw	counter				; counter already 0 ?
				btfsc	STATUS,Z
				goto	CounterUnder		; counter underflow
				decf	counter,f			; decreases counter
				return
CounterUnder	movfw	counter_max
				movwf	counter				; sets counter to max
				return

;** SWITCH ONE CHECK (RB7) -> 0 = NO CHANGE / 1 = CHANGE

SwitchOne		btfss	SW1					; last status of switch high ?
				goto	CheckUpOne			; checks raising edge
				btfss	PORTB,7				; current status of switch high ?
				goto	SwChDownOne			; goto falling edge
				retlw	0x00				; no port change
CheckUpOne		btfsc	PORTB,7				; current status of switch low ?
				goto	SwChUpOne			; goto raising edge
				retlw	0x00				; no port change
SwChDownOne		bcf		SW1					; saves current status
				movfw	sw1_deb_count
				btfss	STATUS,Z
				retlw	0x00				; port change, but no reaction
				movlw	SW_deb_mark			; sets debounce marker
				movwf	sw1_deb_count
				retlw	0x01				; port change -> reaction needed
SwChUpOne		bsf		SW1					; saves current status
				movfw	sw1_deb_count
				btfss	STATUS,Z
				retlw	0x00				; port change, but no reaction
				movlw	SW_deb_mark			; sets debounce marker
				movwf	sw1_deb_count
				retlw	0x00				; port change

;** SWITCH TWO CHECK (RB6) -> 0 = NO CHANGE / 1 = CHANGE

SwitchTwo		btfss	SW2					; last status of switch high ?
				goto	CheckUpTwo			; checks raising edge
				btfss	PORTB,6				; current status of switch high ?
				goto	SwChDownTwo			; goto falling edge
				retlw	0x00				; no port change
CheckUpTwo		btfsc	PORTB,6				; current status of switch low ?
				goto	SwChUpTwo			; goto raising edge
				retlw	0x00				; no port change
SwChDownTwo		bcf		SW2					; saves current status
				movfw	sw2_deb_count
				btfss	STATUS,Z
				retlw	0x00				; port change, but no reaction
				movlw	SW_deb_mark			; sets debounce marker
				movwf	sw2_deb_count
				retlw	0x01				; port change -> reaction needed
SwChUpTwo		bsf		SW2					; saves current status
				movfw	sw2_deb_count
				btfss	STATUS,Z
				retlw	0x00				; port change, but no reaction
				movlw	SW_deb_mark			; sets debounce marker
				movwf	sw2_deb_count
				retlw	0x00				; port change

;***** CHECKING LIGHT SENSOR

ChkLight		btfss	PIR1,CMIF
				return
				bcf		PIR1,CMIF
				btfss	CMCON,C2OUT			; light sensor input (0 = light)
				goto	ChkLightMode
				bsf		RE_trigger_flag		; change happend (no light)
				return
ChkLightMode	movfw	light_mode
				btfss	STATUS,Z
				bsf		RE_trigger_flag		; change happend (outdoor mode)
				return

;***** OBJECT COUNTING

;** DECREASING COUNTER

DecObj			movfw	obj_count_e			; checks 1st digit value
				btfsc	STATUS,Z
				goto	DecTen				; if zero go to 2nd digit
				decf	obj_count_e,f

				movfw	obj_count_e			; 1st digit value zero ?
				btfss	STATUS,Z
				return
				movfw	obj_count_z			; 2nd digit value zero ?
				btfss	STATUS,Z
				return
				movfw	obj_count_r			; rest counter value zero ?
				btfss	STATUS,Z
				return
				bcf		obj_state			; counter change 1 -> 0
				bsf		RE_trigger_flag		; change happend
				return

DecTen			movlw	d'9'				; sets 1st digit to 9
				movwf	obj_count_e
				movfw	obj_count_z			; checks 2nd digit value
				btfsc	STATUS,Z
				goto	DecRest				; if zero go to rest counter
				decf	obj_count_z,f
				return
DecRest			movlw	d'9'				; sets 2nd digit to 9
				movwf	obj_count_z
				decf	obj_count_r,f
				movlw	d'255'
				subwf	obj_count_r,w
				btfss	STATUS,Z			; underflow ?
				return
				incf	obj_count_r,f		; goes back to last value (no underflow)
				clrf	obj_count_z
				clrf	obj_count_e
				return

;** INCREASING COUNTER

IncObj			movlw	d'9'				; checks 1st digit value
				subwf	obj_count_e,w
				btfsc	STATUS,Z
				goto	IncTen				; if 9 go to 2nd digit
				incf	obj_count_e,f

				btfsc	obj_state			; counter change 0 -> 1 ?
				return
				bsf		obj_state			; counter > 0 flag
				bsf 	RE_trigger_flag		; change happend
				return

IncTen			clrf	obj_count_e			; sets 1st digit to zero
				movlw	d'9'				; checks 2nd digit value
				subwf	obj_count_z,w
				btfsc	STATUS,Z
				goto	IncRest				; if 9 goto rest counter
				incf	obj_count_z,f
				return
IncRest			clrf	obj_count_z			; sets 2nd digit to zero
				incf	obj_count_r,f
				movfw	obj_count_r
				btfss	STATUS,Z			; overflow ?
				return
				decf	obj_count_r,f		; goes back to last value (no overflow)
				movlw	d'9'
				movwf	obj_count_z
				movwf	obj_count_e
				return

;***** RELAIS

ChkRelais		btfsc	RE_trigger_flag		; change happend ?
				goto	RelaisState			; yes, check relais state			
				btfss	RE_delay_flag		; relais in delay mode ?
				return
				movfw	re_delay_sec
				btfss	STATUS,Z			; delay sec over ?
				return
				movfw	re_delay_min		; delay min over ?
				btfss	STATUS,Z
				return
				bcf		PORTA,3				; switches relais off
				bcf		RE_delay_flag		; no delay mode
				return

RelaisState		bcf		RE_trigger_flag		; resets trigger flag
				btfsc	PORTA,3				; relais on ?
				goto	RelaisIsOn
				btfss	obj_state			; obj counter > 0 ?
				return
				btfss	CMCON,C2OUT			; light sensor input (0 = light)
				return
				bsf		PORTA,3				; sets relais on
				return

RelaisIsOn		btfss	obj_state			; obj counter > 0 ?
				goto	RelaisDelay
				btfss	CMCON,C2OUT			; light sensor input (0 = light)
				goto	RelaisDelay
				btfss	RE_delay_flag		; relais in delay mode
				return
				bcf		RE_delay_flag		; stops delay mode
				clrf	re_delay_sec		; clears delay counter min + sec
				clrf	re_delay_min
				return

RelaisDelay		movfw	delay_sec			; gets delay secs
				btfss	STATUS,Z			; relais delay sec > 0 ?
				goto	StartDelay			; delay
				movfw	delay_min
				btfss	STATUS,Z			; relais delay min > 0 ?
				goto	StartDelay			; delay
				bcf		PORTA,3				; no delay -> sets relais off
				return
				
StartDelay		movfw	delay_min
				movwf	re_delay_min		; sets up delay min counter
				movfw	delay_sec
				movwf	re_delay_sec		; sets up delay sec counter
				incf	re_delay_sec,f		; buffer for gone ms
				bsf		RE_delay_flag		; sets delay flag -> delay on
				movfw	re_delay_min		; gets delay min
				btfsc	STATUS,Z			; min > 0 -> correction of gone sec
				return
				movfw	m1_scaler
				sublw	d'60'				; gets already gone sec
				addwf	re_delay_sec,f		; adds alreday gone sec to delay sec counter
				return	

;***** EEPROM

;** READ EEPROM

ReadEE			bcf		STATUS,IRP			; select bank 0 for indirect adressing
				movlw	0x6a				; adress in RAM
				movwf	FSR
				BANK1						; select bank 1
				movlw	0x00				; EE read adress is 0x00
				movwf	EEADR
NextByte		bsf		EECON1,RD			; starts reading
				movfw	EEDATA
				movwf	INDF				; writes result to RAM
				incf	FSR,f				; next RAM adress
				incf	EEADR,f				; next EEPROM adress
				movlw	0x6f
				subwf	FSR,w
				btfss	STATUS,Z			; reading finished ?
				goto	NextByte
				BANK0						; select bank 0
				return

;** WRITE EEPROM (ONLY CHANGED VALUES)

WriteEE			bcf		STATUS,IRP			; select bank 0 for indirect adressing
				movlw	0x6a				; adress in RAM
				movwf	FSR
				BANK1						; select bank 1
				movlw	0x00				; EE read adress is 0x00
				movwf	EEADR
NextRead		bsf		EECON1,RD			; starts reading
				movfw	INDF				; reads RAM value
				subwf	EEDATA,f
				btfss	STATUS,Z			; value changed ?
				call	WriteValue			; new value -> write to EE
				incf	FSR,f				; next RAM adress
				incf	EEADR,f				; next EEPROM adress
				movlw	0x6f
				subwf	FSR,w
				btfss	STATUS,Z			; reading finished ?
				goto	NextRead
				BANK0						; select bank 0
				return

WriteValue		movwf	EEDATA				; data from RAM
				bsf		EECON1,WREN			; enables write
				bcf		INTCON,GIE			; disables ints
				movlw	0x55				; sequence
				movwf	EECON2
				movlw	0xaa
				movwf	EECON2
				bsf		EECON1,WR			; starts writing
				bsf		INTCON,GIE			; enables global ints
EEWait			btfsc	EECON1,WR			; finished ?
				goto	EEWait
				decf	FSR,f				; decreases adress to read again and verify
				decf	EEADR,f
				return

;** EEPROM ERROR (WRITING DEFAULT VALUES)

ErrorEE			BANK1
				btfss	EECON1,WRERR		; Error (MCLR) ?
				goto	NoError
				bcf		EECON1,WRERR		; clears error flag
				BANK0
				movlw	d'4'				; default values
				movwf	config_level
				movlw	d'18'
				movwf	sensitivity
				clrf	delay_sec
				clrf	delay_min
				clrf	light_mode
				call	WriteEE				; writes default values
				return
NoError			BANK0
				return

;***** INIT

Init			BANK0						; selects bank 0
				clrf	PORTB				; clears PORTB
				clrf	PORTA				; clears PORTA
				movlw	b'00000101'
				movwf	CMCON				; one independent comparator
				movlw	b'00000001'
				movwf	T1CON				; sets up TMR1
				movlw	0xf4				; sets compare to 500
				movwf	CCPR1L
				movlw	0x01
				movwf	CCPR1H
				movlw	b'00001011'			; sets up compare mode
				movwf	CCP1CON
				BANK1						; selects bank 1
				clrf	OPTION_REG			; sets OPTION register, prescaler is 2, int. pull-ups on
				movlw	b'11000000'
				movwf	INTCON				; sets INTCON register
				movlw	b'00000100'			; sets compare int
				movwf	PIE1
				bsf		PCON,OSCF			; sets internal oscillator frequency to 4 MHz
				movlw	b'11110000'			; sets RB4/5/6/7 as input
				movwf	TRISB				; sets port directions for PORTB
				movlw	b'00100110'			; RA5 is always input, sets RA1, RA2 as input
				movwf	TRISA				; sets port directions for PORTA
				BANK0						; selects bank 0

				call	ErrorEE				; checks for EE errors caused by MCRL
				call	ReadEE				; reads EE values

				clrf	general_a			; resets general reg
				clrf	general_b
				bsf		SW1					; sets last switch status to high (RB6/7)
				bsf		SW2
				clrf	TMR1H				; resets TMR1
				clrf	TMR1L
				clrf	LS0_count			; resets LS counting vars
				clrf	LS1_count
				clrf	obj_count_e			; resets object counter
				clrf	obj_count_z
				clrf	obj_count_r

				movlw	d'8'
				movwf	ms4_scaler			; sets 4ms scaler
				movlw	d'250'
				movwf	s1_scaler			; sets 1s scaler
				movlw	d'60'
				movwf	m1_scaler			; sets 1min scaler
				movlw	d'5'
				movwf	ls_count			; sets LS count intervall

				clrf	track
				retfie

;***** DELAY 0,5s

Delay05s		movlw	d'125'				; 125 x 4ms = 0,5s
				movwf	delay_05s			; sets counter
DelayWait		movfw	delay_05s
				btfss	STATUS,Z			; counter > 0 ?
				goto	DelayWait
				return

;***** MAIN

Main			call	Init
				BANK1						; selects bank 1
				btfsc	PCON,NOT_POR		; 1 = MCLR -> config mode
				goto	MainConf
				bsf		PCON,NOT_POR		; sets NOT_POR to make MCLR check possible
				BANK0						; selects bank 0
				movlw	d'4'
				movwf	config_level		; resets config level
				call	WriteEE

Loop			call	ChkSwitches			; checks switches (RB6 + RB7)
				call	ChkLight			; checks light sensor
				call	ChkRelais			; checks relais state

				movfw	general_a
				andlw	b'00000011'			; get LS0 and LS1	
				movwf	state				; save current LS value
				sublw	b'00000011'			; LS0 and LS1 is broken ?
				btfsc	STATUS,Z			; no information to save
				goto	Loop				; start again

				movfw	track
				andlw	b'00000011'			; get last state of track
				subwf	state,w				; old balue (wreg) == new value (state) ?
				btfsc	STATUS,Z
				goto	Loop				; start again

				movfw	track
				andlw	b'00001100'			; get state before last state
				movwf	track_part
				bcf		STATUS,C
				rrf		track_part,f		; right position (0000xx00 -> 000000xx)
				rrf		track_part,f
				movfw	track_part
				subwf	state,w				; value before old value (wreg) == new value (state) ?
				btfss	STATUS,Z
				goto	NewTrackValue		; new value is really nes
				bcf		STATUS,C
				rrf		track,f				; one step back in track
				bcf		STATUS,C
				rrf		track,f
				goto	Loop				; start again

NewTrackValue	rrf		state,f				; add new value (state) to track
				rrf		state,f
				rlf		track,f
				rlf		state,f
				rlf		track,f

				movlw	seq_a
				subwf	track,w				; sequence a found ?
				btfss	STATUS,Z
				goto	SequB				; check sequence b
				clrf	track				; clear last track
				call	IncObj				; increases object counter
				call	ChkRelais			; checks relais state
				call	Delay05s			; waits 0,5s
				goto	Loop				; start again

SequB			movlw	seq_b
				subwf	track,w				; sequence b found ?
				btfss	STATUS,Z
				goto	Loop				; start again
				clrf	track				; clear last track
				call	DecObj				; decreases object counter
				call	ChkRelais			; checks relais state
				call	Delay05s			; waits 0,5s
				goto	Loop				; start again

;***** MAIN (CONFIGURATION)

MainConf		BANK0						; selects bank 0
				decfsz	config_level,f		; decreases config level
				goto	ConfigLevel			; config level == 0 -> go to normal mode
				BANK1						; selects bank 1
				bcf		PCON,NOT_POR		; deletes NOT_POR to go back to normal mode
				BANK0						; selects bank 0
				movlw	d'04'				; resets config level
				movwf	config_level
ConfigLevel		call	WriteEE				; saves new config level to EEPROM
				bsf		config_mode			; sets config mode flag
				movlw	0x02
				movwf	PCLATH				; go to current config level
				movfw	config_level
				bsf		start_up			; sets startup mode
				addwf	PCL,f				; starts with a and ends with d
				nop	
				goto	DelaySec			; c
				goto	DelayMin			; b
				goto	Sensitivity			; a	
				goto	LightMode			; d

DelaySec		movlw	d'59'
				movwf	counter_max			; sets counter maximum
				movlw	char_d				; shows 'dS'
				movwf	left
				movlw	char_S
				movwf	right
				goto	GetInput

DelayMin		movlw	d'99'
				movwf	counter_max			; sets counter maximum
				movlw	char_d				; shows 'dE'
				movwf	left
				movlw	char_E
				movwf	right
				goto	GetInput

Sensitivity		movlw	d'20'
				movwf	counter_max			; sets counter maximum
				movlw	char_S				; shows 'SE'
				movwf	left
				movlw	char_E
				movwf	right
				goto	GetInput

LightMode		movlw	d'1'
				movwf	counter_max			; sets counter maximum
				movlw	char_L				; shows 'LI'
				movwf	left
				movlw	char_I
				movwf	right
				
GetInput		call	Delay05s			; waits 2s
				call	Delay05s
				call	Delay05s
				call	Delay05s
				bcf		start_up			; stops startup mode
ConfLoop		bcf		STATUS,IRP			; select bank 0 for indirect adressing
				movlw	0x6a
				addwf	config_level,w		; sets adress concerning config level
				movwf	FSR
				movfw	INDF
				movwf	counter				; copies level value to counter
StateLoop		movfw	counter				; copies counter to state
				movwf	state
				call	ChkSwitches			; checks switches
				movfw	state
				subwf	counter,w			; change of counter ?
				btfsc	STATUS,Z
				goto 	StateLoop			; no change of counter
				movfw	counter
				movwf	INDF				; copies counter to level value
				call	WriteEE				; saves new value to EEPROM
				goto	ConfLoop

;***** INITIALIZE EEPROM LOCATIONS

EE				CODE	0x2100

;						config_level, delay_sec, delay_min, sensitivity, light_mode 
				DE		0x04,			0x00,	0x00,		0x12,		0x00

				END							; directive 'end of program'

